<?php

/**
 * Wireless AP controller.
 *
 * @category   Apps
 * @package    Wireless_AP
 * @subpackage Views
 * @author     Todd Lewis <todd@grizzlystar.com> 
 * @copyright  2019 Todd Lewis
 * @license    GPLv3
 */

///////////////////////////////////////////////////////////////////////////////
// C L A S S
///////////////////////////////////////////////////////////////////////////////

/**
 * Wireless AP controller.
 *
 * @category   Apps
 * @package    Wireless_AP
 * @subpackage Controllers
 * @author     Todd Lewis <todd@grizzlystar.com> 
 * @copyright  2019 Todd Lewis
 * @license    GPLv3
 */

class Settings extends ClearOS_Controller
{

    function index()
    {
        $this->_common('view');
    }

    function view()
    {
        $this->_common('view');
    }

    function edit($device)
    {
        $this->_common('edit',$device);
    }

    function delete($device = NULL)
    {
        $this->load->library('wireless_ap/Hostapd');
        $this->hostapd->delete_interface($device);
        redirect('/wireless_ap/wireless_ap');
    }

    function _common($form_type,$device)
    {

        // Load dependencies
        //------------------
        $this->lang->load('wireless_ap');
        $this->load->library('wireless_ap/Hostapd');

        // Set validation rules
        //---------------------
        
        if ( $this->input->post('submit') ) {
            
            // $this->form_validation->set_policy('mode', 'wireless_ap/hostapd', 'validate_wpa_key_management');
            
            $this->form_validation->set_policy('ssid', 'wireless_ap/hostapd', 'validate_ssid');
            $this->form_validation->set_policy('wpa_passphrase', 'wireless_ap/hostapd', 'validate_wpa_passphrase');

            // $this->form_validation->set_policy('interface', 'wireless_ap/hostapd', 'validate_interface');
            // $this->form_validation->set_policy('hw_mode', 'wireless_ap/Hostapd', 'validate_hw_mode');
            // $this->form_validation->set_policy('country_code', 'wireless_ap/Hostapd', 'validate_country_code');
            // $this->form_validation->set_policy('channel', 'wireless_ap/Hostapd', 'validate_channel');
            // $this->form_validation->set_policy('vht_oper_chwidth', 'wireless_ap/Hostapd', 'validate_vht_oper_chwidth');
            // $this->form_validation->set_policy('acs_exclude_dfs', 'wireless_ap/Hostapd', 'validate_acs_exclude_dfs');
            // $this->form_validation->set_policy('ieee80211n', 'wireless_ap/Hostapd', 'validate_ieee80211n');
            // $this->form_validation->set_policy('require_ht', 'wireless_ap/Hostapd', 'validate_require_ht');
            // $this->form_validation->set_policy('require_vht', 'wireless_ap/Hostapd', 'validate_require_vht');
            // $this->form_validation->set_policy('ieee80211ac', 'wireless_ap/Hostapd', 'validate_ieee80211ac');
            // $this->form_validation->set_policy('uapsd_advertisement_enabled', 'wireless_ap/Hostapd', 'validate_uapsd_advertisement_enabled');
            // $this->form_validation->set_policy('spectrum_mgmt_required', 'wireless_ap/Hostapd', 'validate_spectrum_mgmt_required');
            // $this->form_validation->set_policy('local_pwr_constraint', 'wireless_ap/Hostapd', 'validate_local_pwr_constraint');
            // $this->form_validation->set_policy('wmm_enabled', 'wireless_ap/Hostapd', 'validate_wmm_enabled');
            // $this->form_validation->set_policy('max_num_sta', 'wireless_ap/Hostapd', 'validate_max_num_sta');
            // $this->form_validation->set_policy('disassoc_low_ack', 'wireless_ap/Hostapd', 'validate_disassoc_low_ack');
            
            $form_ok = $this->form_validation->run();
        }

        // Save New Configuration file
        //---------------------

        if ( ($this->input->post('submit') && $form_ok) ) {
            try {
                $hostapd                                        = $this->hostapd->get_device_hostapd_settings($this->input->post('device'));
                $iw_info                                        = $this->hostapd->get_device_iw_info($this->input->post('device'));
                $bandwidth_g                                    = $this->input->post('bandwidth_g');
                $bandwidth_a                                    = $this->input->post('bandwidth_a');
                $hostapd['interface']                           = $this->input->post('device');
                
                if(!isset($hostapd['driver']))                  $hostapd['driver']                  = 'nl80211';
                if(!isset($hostapd['ctrl_interface']))          $hostapd['ctrl_interface']          = '/var/run/hostapd';
                if(!isset($hostapd['ctrl_interface_group']))    $hostapd['ctrl_interface_group']    = '0';
                if(!isset($hostapd['wds_sta']))                 $hostapd['wds_sta']                 = '1'; //allows associated stations to use 4-address frames to allow layer 2
                
                $hostapd['ssid']                                = $this->input->post('ssid');
                $hostapd['hw_mode']                             = $this->input->post('hw_mode');
                $hostapd['channel']                             = ''; 
                $hostapd['chanlist']                            = ''; 
                $hostapd['channel_g']                           = $this->input->post('channel_g');
                $hostapd['channel_a']                           = $this->input->post('channel_a');
                $hostapd['channel_dfs']                         = $this->input->post('channel_dfs');
                $hostapd['max_num_sta']                         = $this->input->post('max_num_sta');
                $hostapd['disassoc_low_ack']                    = $this->input->post('disassoc_low_ack');
                
                // if(!isset($hostapd['auth_algs']))               $hostapd['auth_algs']    = '1'; // 1=Only allow WPA , should only be used with open systems. 
                if(!isset($hostapd['wpa']))                     $hostapd['wpa']                      = $this->input->post('wpa'); // 1 = WPA , 2 = WPA2 , 3 = Both
                
                $hostapd['wpa_pairwise']                        = 'CCMP';
                if( $hostapd['wpa'] = '3')                      $hostapd['wpa_pairwise']              = 'TKIP CCMP';

                $hostapd['wpa_key_mgmt']                        = 'WPA-PSK WPA-EAP WPA-PSK-SHA256 WPA-EAP-SHA256'; //adding every keymanagement doesn't hurt anything. 
                $hostapd['wpa_passphrase']                      = $this->input->post('wpa_passphrase');
                
                $hostapd['country_code']                        = $this->input->post('country_code');
                $hostapd['acs_exclude_dfs']                     = $this->input->post('acs_exclude_dfs');
                             
                //if(!isset($hostapd['ht_capab']))                
                    $hostapd['ht_capab']                   = $this->hostapd->get_ht_capab($iw_info);
                //if(!isset($hostapd['vht_capab']))
                    $hostapd['vht_capab']                  = $this->hostapd->get_vht_capab($iw_info);

                $hostapd['ieee80211n']                          = '0';
                if (strpos($hostapd['ht_capab'],"40"))          $hostapd['ieee80211n']                  = '1';

                $hostapd['ieee80211ac']                         = '0';
                $hostapd['vht_oper_chwidth']                    = '0';
                
                if ( $bandwidth_g == 20 ){
                    $hostapd['ht_capab'] = str_replace('[HT40+]',' ',$hostapd['ht_capab']);
                    $hostapd['ht_capab'] = str_replace('[SHORT-GI-40]',' ',$hostapd['ht_capab']);
                    $hostapd['ht_capab'] = str_replace('[DSSS_CCK-40]',' ',$hostapd['ht_capab']);

                } 

                if ( $hostapd['hw_mode'] === 'a' ) {
                    
                    if ( $bandwidth_a == 20 ){
                        $hostapd['ht_capab'] = str_replace('[HT40+]',' ',$hostapd['ht_capab']);
                        $hostapd['ht_capab'] = str_replace('[SHORT-GI-40]',' ',$hostapd['ht_capab']);
                        $hostapd['ht_capab'] = str_replace('[DSSS_CCK-40]',' ',$hostapd['ht_capab']);
    
                    } 
                    
                    if( strpos( $hostapd['vht_capab'],'80' ) )  $hostapd['ieee80211ac']                     = '1';
                    if ( $bandwidth_a >= '80' )                 $hostapd['vht_oper_chwidth']                = '1';
                }

                $hostapd['uapsd_advertisement_enabled']         = $this->input->post('uapsd_advertisement_enabled');
                $hostapd['wmm_enabled']                         = $this->input->post('wmm_enabled');
                if( $hostapd['ieee80211n'] == '0')               $hostapd['wmm_enabled']              = '0';  // wmm requires 802.11n
                
                if(!isset($hostapd['wmm_ac_bk_cwmin']))         $hostapd['wmm_ac_bk_cwmin']       = '4';
                if(!isset($hostapd['wmm_ac_bk_cwmax']))         $hostapd['wmm_ac_bk_cwmax']       = '10';
                if(!isset($hostapd['wmm_ac_bk_aifs']))          $hostapd['wmm_ac_bk_aifs']        = '7';
                if(!isset($hostapd['wmm_ac_bk_txop_limit']))    $hostapd['wmm_ac_bk_txop_limit']  = '0';
                if(!isset($hostapd['wmm_ac_bk_acm']))           $hostapd['wmm_ac_bk_acm']         = '0';
                // Normal priority / AC_BE '] best effort
                if(!isset($hostapd['wmm_ac_be_aifs']))          $hostapd['wmm_ac_be_aifs']        = '3';
                if(!isset($hostapd['wmm_ac_be_cwmin']))         $hostapd['wmm_ac_be_cwmin']       = '4';
                if(!isset($hostapd['wmm_ac_be_cwmax']))         $hostapd['wmm_ac_be_cwmax']       = '10';
                if(!isset($hostapd['wmm_ac_be_txop_limit']))    $hostapd['wmm_ac_be_txop_limit']  = '0';
                if(!isset($hostapd['wmm_ac_be_acm']))           $hostapd['wmm_ac_be_acm']         = '0';
                // High priority / AC_VI '] =  video              
                if(!isset($hostapd['wmm_ac_vi_aifs']))          $hostapd['wmm_ac_vi_aifs']        = '2';
                if(!isset($hostapd['wmm_ac_vi_cwmin']))         $hostapd['wmm_ac_vi_cwmin']       = '3';
                if(!isset($hostapd['wmm_ac_vi_cwmax']))         $hostapd['wmm_ac_vi_cwmax']       = '4';
                if(!isset($hostapd['wmm_ac_vi_txop_limit']))    $hostapd['wmm_ac_vi_txop_limit']  = '9';
                if(!isset($hostapd['wmm_ac_vi_acm']))           $hostapd['wmm_ac_vi_acm']         = '0';
                # Highest priority / AC_VO '] =  voice
                if(!isset($hostapd['wmm_ac_vo_aifs']))          $hostapd['wmm_ac_vo_aifs']        = '2';
                if(!isset($hostapd['wmm_ac_vo_cwmin']))         $hostapd['wmm_ac_vo_cwmin']       = '2';
                if(!isset($hostapd['wmm_ac_vo_cwmax']))         $hostapd['wmm_ac_vo_cwmax']       = '3';
                if(!isset($hostapd['wmm_ac_vo_txop_limit']))    $hostapd['wmm_ac_vo_txop_limit']  = '47';
                if(!isset($hostapd['wmm_ac_vo_acm']))           $hostapd['wmm_ac_vo_acm']         = '0';
                // TX queue parameters
                if(!isset($hostapd['tx_queue_data3_aifs']))     $hostapd['tx_queue_data3_aifs']   = '7';
                if(!isset($hostapd['tx_queue_data3_cwmin']))    $hostapd['tx_queue_data3_cwmin']  = '15';
                if(!isset($hostapd['tx_queue_data3_cwmax']))    $hostapd['tx_queue_data3_cwmax']  = '1023';
                if(!isset($hostapd['tx_queue_data3_burst']))    $hostapd['tx_queue_data3_burst']  = '0';
                if(!isset($hostapd['tx_queue_data2_aifs']))     $hostapd['tx_queue_data2_aifs']   = '3';
                if(!isset($hostapd['tx_queue_data2_cwmin']))    $hostapd['tx_queue_data2_cwmin']  = '15';
                if(!isset($hostapd['tx_queue_data2_cwmax']))    $hostapd['tx_queue_data2_cwmax']  = '63';
                if(!isset($hostapd['tx_queue_data2_burst']))    $hostapd['tx_queue_data2_burst']  = '0';
                if(!isset($hostapd['tx_queue_data1_aifs']))     $hostapd['tx_queue_data1_aifs']   = '1';
                if(!isset($hostapd['tx_queue_data1_cwmin']))    $hostapd['tx_queue_data1_cwmin']  = '7';
                if(!isset($hostapd['tx_queue_data1_cwmax']))    $hostapd['tx_queue_data1_cwmax']  = '15';
                if(!isset($hostapd['tx_queue_data1_burst']))    $hostapd['tx_queue_data1_burst']  = '3.0';
                if(!isset($hostapd['tx_queue_data0_aifs']))     $hostapd['tx_queue_data0_aifs']   = '1';
                if(!isset($hostapd['tx_queue_data0_cwmin']))    $hostapd['tx_queue_data0_cwmin']  = '3';
                if(!isset($hostapd['tx_queue_data0_cwmax']))    $hostapd['tx_queue_data0_cwmax']  = '7';
                if(!isset($hostapd['tx_queue_data0_burst']))    $hostapd['tx_queue_data0_burst']  = '1.5';
                
                $hostapd['g']['001']                        = $this->input->post('g_001');
                $hostapd['g']['002']                        = $this->input->post('g_002');
                $hostapd['g']['003']                        = $this->input->post('g_003');
                $hostapd['g']['004']                        = $this->input->post('g_004');
                $hostapd['g']['005']                        = $this->input->post('g_005');
                $hostapd['g']['006']                        = $this->input->post('g_006');
                $hostapd['g']['007']                        = $this->input->post('g_007');
                $hostapd['g']['008']                        = $this->input->post('g_008');
                $hostapd['g']['009']                        = $this->input->post('g_009');
                $hostapd['g']['010']                        = $this->input->post('g_010');
                $hostapd['g']['011']                        = $this->input->post('g_011');
                $hostapd['g']['012']                        = $this->input->post('g_012');
                $hostapd['g']['013']                        = $this->input->post('g_013');
                $hostapd['g']['014']                        = $this->input->post('g_014');

                $hostapd['a']['036']                        = $this->input->post('a_036');
                $hostapd['a']['040']                        = $this->input->post('a_040');
                $hostapd['a']['044']                        = $this->input->post('a_044');
                $hostapd['a']['048']                        = $this->input->post('a_048');
                $hostapd['a']['052']                        = $this->input->post('a_052');
                $hostapd['a']['056']                        = $this->input->post('a_056');
                $hostapd['a']['060']                        = $this->input->post('a_060');
                $hostapd['a']['064']                        = $this->input->post('a_064');
                $hostapd['a']['100']                        = $this->input->post('a_100');
                $hostapd['a']['104']                        = $this->input->post('a_104');
                $hostapd['a']['108']                        = $this->input->post('a_108');
                $hostapd['a']['112']                        = $this->input->post('a_112');
                $hostapd['a']['116']                        = $this->input->post('a_116');
                $hostapd['a']['120']                        = $this->input->post('a_120');
                $hostapd['a']['124']                        = $this->input->post('a_124');
                $hostapd['a']['128']                        = $this->input->post('a_128');
                $hostapd['a']['132']                        = $this->input->post('a_132');
                $hostapd['a']['136']                        = $this->input->post('a_136');
                $hostapd['a']['140']                        = $this->input->post('a_140');
                $hostapd['a']['144']                        = $this->input->post('a_144');
                $hostapd['a']['149']                        = $this->input->post('a_149');
                $hostapd['a']['153']                        = $this->input->post('a_153');
                $hostapd['a']['157']                        = $this->input->post('a_157');
                $hostapd['a']['161']                        = $this->input->post('a_161');
                $hostapd['a']['165']                        = $this->input->post('a_165');
                
                $hostapd['dfs']['036']                      = $this->input->post('dfs_036');
                $hostapd['dfs']['040']                      = $this->input->post('dfs_040');
                $hostapd['dfs']['044']                      = $this->input->post('dfs_044');
                $hostapd['dfs']['048']                      = $this->input->post('dfs_048');
                $hostapd['dfs']['052']                      = $this->input->post('dfs_052');
                $hostapd['dfs']['056']                      = $this->input->post('dfs_056');
                $hostapd['dfs']['060']                      = $this->input->post('dfs_060');
                $hostapd['dfs']['064']                      = $this->input->post('dfs_064');
                $hostapd['dfs']['100']                      = $this->input->post('dfs_100');
                $hostapd['dfs']['104']                      = $this->input->post('dfs_104');
                $hostapd['dfs']['108']                      = $this->input->post('dfs_108');
                $hostapd['dfs']['112']                      = $this->input->post('dfs_112');
                $hostapd['dfs']['116']                      = $this->input->post('dfs_116');
                $hostapd['dfs']['120']                      = $this->input->post('dfs_120');
                $hostapd['dfs']['124']                      = $this->input->post('dfs_124');
                $hostapd['dfs']['128']                      = $this->input->post('dfs_128');
                $hostapd['dfs']['132']                      = $this->input->post('dfs_132');
                $hostapd['dfs']['136']                      = $this->input->post('dfs_136');
                $hostapd['dfs']['140']                      = $this->input->post('dfs_140');
                $hostapd['dfs']['144']                      = $this->input->post('dfs_144');
                $hostapd['dfs']['149']                      = $this->input->post('dfs_149');
                $hostapd['dfs']['153']                      = $this->input->post('dfs_153');
                $hostapd['dfs']['157']                      = $this->input->post('dfs_157');
                $hostapd['dfs']['161']                      = $this->input->post('dfs_161');
                $hostapd['dfs']['165']                      = $this->input->post('dfs_165');


                $this->hostapd->set_hostapd($hostapd,$iw_info);
                $this->page->set_status_updated();
                redirect('/wireless_ap/wireless_ap');

            } catch (Exception $e) {
                $this->page->view_exception($e);
                return;
            }
        }

        // Load view data
        //---------------
        
        if ( $form_type == 'view' ){

            $data['form_type'] = $form_type ;
            
            try {
                // get hostapd setttings
                $wifi_devices = $this->hostapd->get_wifi_devices();
                foreach ( $wifi_devices as $device) {
                    $data['wifi_devices'][$device]              = $this->hostapd->get_device_network_settings($device);
                    $data['wifi_devices'][$device]['hostapd']   = $this->hostapd->get_device_hostapd_settings($device);
                }
       
                } catch (Exception $e) {
                    $this->page->view_exception($e);
                return;
            }  
        }
        
        // Load edit data
        //---------------
        if ( $form_type == 'edit' ){

            //get device info
            $device_iw_info = $this->hostapd->get_device_iw_info($device);

            $data['form_type']          = $form_type ;
            $data['device']             = $device ; 
            $data['network_settings']   = $this->hostapd->get_device_network_settings($device);
            $data['general_info']       = $this->hostapd->get_device_general_info($device_iw_info);
            $data['get_dfs_status']     = $this->hostapd->get_dfs_status();
            $data['channels']           = $this->hostapd->get_device_channels($device_iw_info);
            $data['options']            = $this->hostapd->get_hostapd_options($device_iw_info);
            $data['hostapd']            = $this->hostapd->get_device_hostapd_settings($device);

            $data['hostapd']['bandwidth_g']                     = 20;
            $data['hostapd']['bandwidth_a']                     = 20;
            
            if ( strpos($data['hostapd']['ht_capab'],"40")){
                $data['hostapd']['bandwidth_g']                 = 40;
                $data['hostapd']['bandwidth_a']                 = 40;
            }
            
            if ( $data['hostapd']['vht_oper_chwidth'] == '1') 
                $data['hostapd']['bandwidth_a']                 = 80;         


            if( $data['hostapd']['hostapd_configured'] == FALSE ) 
                $data['hostapd'] = $this->hostapd->get_default_hostapd_settings($device);
        }
        
        // Load views
        //-----------

        $this->page->view_form( 'wireless_ap/settings', $data, lang('base_settings') );
    }
}
