<?php

/**
 * Wireless AP controller.
 *
 * @category   Apps
 * @package    Wireless_AP
 * @subpackage Controllers
 * @author     Todd Lewis <todd@grizzlystar.com> 
 * @copyright  2019 Todd Lewis
 * @license    GPLv3
 */

///////////////////////////////////////////////////////////////////////////////
// Load dependencies
///////////////////////////////////////////////////////////////////////////////

$this->lang->load('base');
$this->lang->load('wireless_ap');


///////////////////////////////////////////////////////////////////////////////
// Form
///////////////////////////////////////////////////////////////////////////////

echo form_header(lang('wireless_ap_app_name'));


///////////////////////////////////////////////////////////////////////////////
// Title
///////////////////////////////////////////////////////////////////////////////

$title = lang('base_settings');

if( $form_type == 'view') {
    
    $headers = array(
        lang('wireless_ap_iface'),
        lang('wireless_ap_network_type'),
        lang('wireless_ap_ssid'),
    );

    if(isset($wifi_devices)){
        foreach ( $wifi_devices as $device => $device_settings){
     
            // set default buttons
            $buttons = array(
                anchor_edit('/app/wireless_ap/settings/edit/'     . $device),
                anchor_delete('/app/wireless_ap/settings/delete/' . $device)
            );    

            //replace default buttons if device is not configured. 
            if( $device_settings['hostapd']['hostapd_configured'] == FALSE )
                $buttons = array(
                    anchor_add('/app/wireless_ap/settings/edit/' . $device),
                );

            if ( $device_settings['network_configured'] == FALSE ){
                $buttons = array(
                    anchor_add('/app/network/iface/edit/' . $device),
                ); 
                $device_settings['hostapd']['ssid']  = 'Configure Network Settings';
            }


            $item['title'] = 'Wireless Interfaces';
            $item['action'] = '';
            $item['anchors'] = button_set($buttons);
            $item['details'] = array(
                "<span id='iface_"      . $device . "'>" . $device                                      . "</span>",
                "<span id='type_"       . $device . "'>" . $device_settings['network_type']             . "</span>",
                "<span id='ssid_"       . $device . "'>" . $device_settings['hostapd']['ssid']          . "</span>",
            );

            $items[] = $item;
      
        }

        $options['id'] = 'device_summary';

        echo summary_table(
            $title,
            $anchors,
            $headers,
            $items,
            $options
        );

      
    } else {
        echo fieldset_header('No Wireless Adapters Found');
    }

    echo form_footer();
    echo form_close();

} elseif( $form_type == 'edit'){

    // There is a glitch in the form header. You need to make sure teh div tages are closed. 
    echo "</div>";
    echo "</div>";

    ///////////////////////////////////////////////////////////////////////////////
    // Form handler
    ///////////////////////////////////////////////////////////////////////////////

    $read_only = FALSE;
    $form_path = '/wireless_ap/settings/edit/' . $device;
    $buttons = array(
        form_submit_update('submit'),
        anchor_cancel('/app/wireless_ap'),
    );

    ///////////////////////////////////////////////////////////////////////////////
    // Form
    ///////////////////////////////////////////////////////////////////////////////

    echo form_open($form_path);
    echo form_header(lang('base_settings'));
    echo fieldset_header('Basic Settings');
    echo field_input( 'device', $device, lang('wireless_ap_device'), TRUE);
    echo field_dropdown('hw_mode',$options['hw_mode'], $hostapd['hw_mode'], lang('wireless_ap_hw_mode'), $read_only);
    echo field_dropdown('country_code', $options['country'], $hostapd['country_code'],lang('wireless_ap_country_code'), $read_only); 

    echo field_input('ssid', $hostapd['ssid'], lang('wireless_ap_ssid'), $read_only);
    echo field_dropdown('wpa', $options['wpa'], $hostapd['wpa'], lang('wireless_ap_wpa'), $read_only);
    

    if ($hostapd['wpa_key_mgmt'] == 'WPA-EAP') {
        echo field_input('wpa_passphrase', $hostapd['wpa_passphrase'], lang('wireless_ap_radius_secret'), $read_only);
    } else {
        echo field_input('wpa_passphrase', $hostapd['wpa_passphrase'], lang('wireless_ap_passphrase'), $read_only);
    }

    echo fieldset_header('Radio Channel Settings');
    if ( $get_dfs_status == TRUE ){
        echo field_dropdown('acs_exclude_dfs', $options['acs_exclude_dfs'], $hostapd['acs_exclude_dfs'], lang('wireless_ap_acs_exclude_dfs'), $read_only); 
        echo field_dropdown('spectrum_mgmt_required', $options['spectrum_mgmt_required'], $hostapd['mgmt_required'], lang('wireless_ap_spectrum_mgmt_required'), $read_only); 
        echo field_input('local_pwr_constraint', $hostapd['local_pwr_constraint'],lang('wireless_ap_local_pwr_constraint'), $read_only); 
    }
    
    echo field_simple_dropdown('channel_g', $channels['g'], $hostapd['channel'], lang('wireless_ap_channel'), $read_only);
    echo field_simple_dropdown('channel_a', $channels['a'], $hostapd['channel'], lang('wireless_ap_channel'), $read_only);
    echo field_simple_dropdown('channel_dfs', $channels['dfs'], $hostapd['channel'], lang('wireless_ap_channel'), $read_only);

    echo field_dropdown('bandwidth_g', $options['bandwidth_g'], $hostapd['bandwidth_g'], lang('wireless_ap_bandwidth'), $read_only); 
    echo field_dropdown('bandwidth_a', $options['bandwidth_a'], $hostapd['bandwidth_a'], lang('wireless_ap_bandwidth'), $read_only);

    //echo field_dropdown('ieee80211ac', $options['ieee80211ac'], $hostapd['ieee80211ac'], lang('wireless_ap_ieee80211ac'), $read_only); 
    
    
    //echo field_dropdown('vht_oper_chwidth', $options['vht_oper_chwidth'], $hostapd['vht_oper_chwidth'], lang('wireless_ap_vht_oper_chwidth'), $read_only); 
    //echo field_dropdown('vht_oper_chwidth', $options['vht_oper_chwidth'], $hostapd['vht_oper_chwidth'], lang('wireless_ap_vht_oper_chwidth'), $read_only); 
    //echo field_dropdown('require_ht', $options['require_ht'], $hostapd['require_ht'], lang('wireless_ap_require_ht'), $read_only); 
    //echo field_dropdown('require_vht', $options['require_vht'], $hostapd['require_vht'], lang('wireless_ap_require_vht'), $read_only); 

   

    
    echo "<div id='channel_mutiselect_form' >";
        echo fieldset_header('Limit Automatic Channel Selection to the following Channels');
        $chanlist = explode(' ', $hostapd['chanlist']);
        $chanlist = implode(' , ', $chanlist);
        
        echo field_input('chanlist', $chanlist, lang('wireless_ap_chanlist'), TRUE);
        
        unset($channels['g']['Automatic']);
        unset($channels['a']['Automatic']);
        unset($channels['dfs']['Automatic']);
        
        $channel_mutiselect = array_merge( $channels['g'], $channels['dfs']);
        $channel_mutiselect = $channels['dfs'];

        $select = array (
            '0' => 'g',
            '1' => 'a',
            '2' => 'dfs',
        );

        foreach ($select as $hw_mode){

            echo "<div id='channel_mutiselect_form_$hw_mode' >";
            
            foreach ($channels[$hw_mode] as $key => $value) {
                
                if ( $key < 10 ) {
                    $text = '00' . $key ;
                } elseif ( $key < 99 ) {
                    $text = '0' . $key ;
                } else {
                    $text = $key;
                }
                
                $data = array(
                    'name'          => $hw_mode . "_" . $text,
                    'id'            => $hw_mode . "_" . $text,
                    'value'         => '1',
                    'checked'       => TRUE,
                    'style'         => 'margin:10px;'
                );
                
                    
                //Display Data
                echo "<label id='channel_mutiselect_$text'>";
                echo form_checkbox($data);
                echo $key;
                echo "</label>";
            }

            //channel_mutiselect_form_hw_mode div tag 
            echo "</div>";
        } 

        //channel_mutiselect_form div tag 
    echo "</div>";
    

    echo fieldset_header('Advance Settings');
    // Enable smooth Transitions between accesss points
    echo field_dropdown('uapsd_advertisement_enabled', $options['uapsd_advertisement_enabled'], $hostapd['uapsd_advertisement_enabled'],lang('wireless_ap_uapsd_advertisement_enabled'), $read_only);
    // Enable QoS for WiFi Connection
    echo field_dropdown('wmm_enabled', $options['wmm_enabled'], $hostapd['wmm_enabled'],lang('wireless_ap_wmm_enabled'), $read_only);
    //Max number of stations allowed in station, max 2007
    echo field_input('max_num_sta', $hostapd['max_num_sta'], lang('wireless_ap_max_num_sta') , $read_only); 
    //Disassociate stations based on excessive transmission failures
    echo field_dropdown('disassoc_low_ack', $options['disassoc_low_ack'], $hostapd['disassoc_low_ack'], lang('wireless_ap_disassoc_low_ack'), $read_only); 

    echo field_button_set($buttons);

    echo form_footer();
    echo form_close();
    
    
}

echo "<div id='wireless_ap_form_end' ></div>";